@@ -1,21 +1,23 @@
 use super::common::*;
 use crate::circuit::mixer::MixerCircuit;
+use ark_crypto_primitives::SNARK;
+use ark_ec::PairingEngine;
+use ark_ff::{BigInteger, PrimeField};
+use ark_groth16::{Groth16, Proof, ProvingKey, VerifyingKey};
 use ark_serialize::{CanonicalDeserialize, CanonicalSerialize};
-use ark_std::rc::Rc;
+use ark_std::{
+	rand::{CryptoRng, Rng, RngCore},
+	rc::Rc,
+	vec::Vec,
+};
 use arkworks_gadgets::{
 	arbitrary::mixer_data::Input as MixerDataInput,
 	leaf::mixer::{constraints::MixerLeafGadget, MixerLeaf, Private as LeafPrivate},
 	merkle_tree::Path,
 };
-use arkworks_utils::{poseidon::PoseidonParameters, utils::common::verify_groth16};
-
-use ark_crypto_primitives::SNARK;
-use ark_ec::PairingEngine;
-use ark_ff::{BigInteger, PrimeField};
-use ark_groth16::{Groth16, Proof, ProvingKey, VerifyingKey};
-use ark_std::{
-	rand::{CryptoRng, Rng, RngCore},
-	vec::Vec,
+use arkworks_utils::{
+	poseidon::PoseidonParameters,
+	utils::common::{setup_params_x5_3, setup_params_x5_5, verify_groth16, Curve},
 };
 
 pub type MixerConstraintDataInput<F> = MixerDataInput<F>;
@@ -83,7 +85,7 @@ pub fn setup_leaf<F: PrimeField, R: RngCore>(
 	)
 }
 
-pub fn setup_leaf_with_privates_raw<F: PrimeField, R: RngCore>(
+pub fn setup_leaf_with_privates_raw<F: PrimeField>(
 	params5: &PoseidonParameters<F>,
 	secret_bytes: Vec<u8>,
 	nullfier_bytes: Vec<u8>,
@@ -101,6 +103,49 @@ pub fn setup_leaf_with_privates_raw<F: PrimeField, R: RngCore>(
 	(leaf_bytes, nullifier_hash_bytes)
 }
 
+pub const LEN: usize = 30;
+type MixerProverSetupBn254_30<F> = MixerProverSetup<F, LEN>;
+
+pub fn setup_proof_x5_5<E: PairingEngine, R: RngCore + CryptoRng>(
+	curve: Curve,
+	secret_raw: Vec<u8>,
+	nullifier_raw: Vec<u8>,
+	leaves_raw: Vec<Vec<u8>>,
+	index: u64,
+	recipient_raw: Vec<u8>,
+	relayer_raw: Vec<u8>,
+	fee: u128,
+	refund: u128,
+	pk: Vec<u8>,
+	rng: &mut R,
+) -> (Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>, Vec<Vec<u8>>) {
+	let params3 = setup_params_x5_3::<E::Fr>(curve);
+	let params5 = setup_params_x5_5::<E::Fr>(curve);
+	let prover = MixerProverSetupBn254_30::new(params3, params5);
+
+	let (circuit, leaf_raw, nullifier_hash_raw, root_raw, public_inputs_raw) = prover
+		.setup_circuit_with_privates_raw(
+			secret_raw,
+			nullifier_raw,
+			&leaves_raw,
+			index,
+			recipient_raw,
+			relayer_raw,
+			fee,
+			refund,
+		);
+
+	let proof = MixerProverSetupBn254_30::<E::Fr>::prove_unchecked::<E, _>(circuit, &pk, rng);
+
+	(
+		proof,
+		leaf_raw,
+		nullifier_hash_raw,
+		root_raw,
+		public_inputs_raw,
+	)
+}
+
 pub struct MixerProverSetup<F: PrimeField, const N: usize> {
 	params3: PoseidonParameters<F>,
 	params5: PoseidonParameters<F>,
@@ -357,6 +402,19 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		(pk_bytes, vk_bytes)
 	}
 
+	pub fn setup_keys_unchecked<E: PairingEngine, R: RngCore + CryptoRng>(
+		circuit: Circuit_x5<E::Fr, N>,
+		rng: &mut R,
+	) -> (Vec<u8>, Vec<u8>) {
+		let (pk, vk) = Groth16::<E>::circuit_specific_setup(circuit, rng).unwrap();
+
+		let mut pk_bytes = Vec::new();
+		let mut vk_bytes = Vec::new();
+		pk.serialize_unchecked(&mut pk_bytes).unwrap();
+		vk.serialize_unchecked(&mut vk_bytes).unwrap();
+		(pk_bytes, vk_bytes)
+	}
+
 	pub fn prove<E: PairingEngine, R: RngCore + CryptoRng>(
 		circuit: Circuit_x5<E::Fr, N>,
 		pk_bytes: &[u8],
@@ -370,8 +428,36 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		proof_bytes
 	}
 
-	pub fn verify<E: PairingEngine>(public_inputs: &[E::Fr], vk: &[u8], proof: &[u8]) -> bool {
-		let vk = VerifyingKey::<E>::deserialize(vk).unwrap();
+	pub fn prove_unchecked<E: PairingEngine, R: RngCore + CryptoRng>(
+		circuit: Circuit_x5<E::Fr, N>,
+		pk_unchecked_bytes: &[u8],
+		rng: &mut R,
+	) -> Vec<u8> {
+		let pk = ProvingKey::<E>::deserialize_unchecked(pk_unchecked_bytes).unwrap();
+
+		let proof = Groth16::prove(&pk, circuit, rng).unwrap();
+		let mut proof_bytes = Vec::new();
+		proof.serialize(&mut proof_bytes).unwrap();
+		proof_bytes
+	}
+
+	pub fn verify<E: PairingEngine>(
+		public_inputs: &[E::Fr],
+		vk_bytes: &[u8],
+		proof: &[u8],
+	) -> bool {
+		let vk = VerifyingKey::<E>::deserialize(vk_bytes).unwrap();
+		let proof = Proof::<E>::deserialize(proof).unwrap();
+		let ver_res = verify_groth16(&vk, &public_inputs, &proof);
+		ver_res
+	}
+
+	pub fn verify_unchecked<E: PairingEngine>(
+		public_inputs: &[E::Fr],
+		vk_unchecked_bytes: &[u8],
+		proof: &[u8],
+	) -> bool {
+		let vk = VerifyingKey::<E>::deserialize_unchecked(vk_unchecked_bytes).unwrap();
 		let proof = Proof::<E>::deserialize(proof).unwrap();
 		let ver_res = verify_groth16(&vk, &public_inputs, &proof);
 		ver_res
